/* knapsack.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "alloc.h"
#include "knapsack.h"


/* Prototipi delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *filedati);

/* Algoritmo ricorsivo per il problema dello zaino */
int AlgoritmoRicorsivoKP (int n, vint phi, vint v, int V);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char filedati[ROW_LENGTH];
  datiKP I;
  soluzioneKP S;
  clock_t inizio, fine;
  double tempo;


  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,filedati);

  /* Carica i dati */
  CaricaDatiKP(filedati,&I);

  /* Crea una soluzione vuota */
  CreaSoluzioneKP(I.n,&S);

  /* Risolve il problema dello zaino */
  inizio = clock();
  S.f = AlgoritmoRicorsivoKP(I.n,I.phi,I.v,I.V);
  fine = clock();
  tempo = ((double) (fine - inizio))/CLOCKS_PER_SEC;

  /* Stampa il tempo di calcolo e la soluzione */
  printf("CPU = %f\n",tempo);
  StampaSoluzioneKP(I.n,&S);

  /* Dealloca le strutture dati dinamiche */
  DistruggeDatiKP(&I);
  DistruggeSoluzioneKP(&S);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *filedati)
{
  if (argc != 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    fprintf(stderr,"%s [filedati]\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  strcpy(filedati,argv[1]);
}


/* Algoritmo ricorsivo per il problema dello zaino */
int AlgoritmoRicorsivoKP (int n, vint phi, vint v, int V)
{
  int f0, f1;
  int f;

  if (n == 0)
    f = 0;
  else if (v[n] > V)
    f = AlgoritmoRicorsivoKP(n-1,phi,v,V);
  else /* (v[n] <= V) */
  {
    f0 = AlgoritmoRicorsivoKP(n-1,phi,v,V);
    f1 = phi[n] + AlgoritmoRicorsivoKP(n-1,phi,v,V-v[n]);
    f = ( (f0 >= f1) ? f0 : f1 );
  }

  return f;
}
